import axios from 'axios';

var isMergeableObject = function isMergeableObject(value) {
	return isNonNullObject(value)
		&& !isSpecial(value)
};

function isNonNullObject(value) {
	return !!value && typeof value === 'object'
}

function isSpecial(value) {
	var stringValue = Object.prototype.toString.call(value);

	return stringValue === '[object RegExp]'
		|| stringValue === '[object Date]'
		|| isReactElement(value)
}

// see https://github.com/facebook/react/blob/b5ac963fb791d1298e7f396236383bc955f916c1/src/isomorphic/classic/element/ReactElement.js#L21-L25
var canUseSymbol = typeof Symbol === 'function' && Symbol.for;
var REACT_ELEMENT_TYPE = canUseSymbol ? Symbol.for('react.element') : 0xeac7;

function isReactElement(value) {
	return value.$$typeof === REACT_ELEMENT_TYPE
}

function emptyTarget(val) {
	return Array.isArray(val) ? [] : {}
}

function cloneUnlessOtherwiseSpecified(value, options) {
	return (options.clone !== false && options.isMergeableObject(value))
		? deepmerge(emptyTarget(value), value, options)
		: value
}

function defaultArrayMerge(target, source, options) {
	return target.concat(source).map(function(element) {
		return cloneUnlessOtherwiseSpecified(element, options)
	})
}

function getMergeFunction(key, options) {
	if (!options.customMerge) {
		return deepmerge
	}
	var customMerge = options.customMerge(key);
	return typeof customMerge === 'function' ? customMerge : deepmerge
}

function getEnumerableOwnPropertySymbols(target) {
	return Object.getOwnPropertySymbols
		? Object.getOwnPropertySymbols(target).filter(function(symbol) {
			return target.propertyIsEnumerable(symbol)
		})
		: []
}

function getKeys(target) {
	return Object.keys(target).concat(getEnumerableOwnPropertySymbols(target))
}

function propertyIsOnObject(object, property) {
	try {
		return property in object
	} catch(_) {
		return false
	}
}

// Protects from prototype poisoning and unexpected merging up the prototype chain.
function propertyIsUnsafe(target, key) {
	return propertyIsOnObject(target, key) // Properties are safe to merge if they don't exist in the target yet,
		&& !(Object.hasOwnProperty.call(target, key) // unsafe if they exist up the prototype chain,
			&& Object.propertyIsEnumerable.call(target, key)) // and also unsafe if they're nonenumerable.
}

function mergeObject(target, source, options) {
	var destination = {};
	if (options.isMergeableObject(target)) {
		getKeys(target).forEach(function(key) {
			destination[key] = cloneUnlessOtherwiseSpecified(target[key], options);
		});
	}
	getKeys(source).forEach(function(key) {
		if (propertyIsUnsafe(target, key)) {
			return
		}

		if (propertyIsOnObject(target, key) && options.isMergeableObject(source[key])) {
			destination[key] = getMergeFunction(key, options)(target[key], source[key], options);
		} else {
			destination[key] = cloneUnlessOtherwiseSpecified(source[key], options);
		}
	});
	return destination
}

function deepmerge(target, source, options) {
	options = options || {};
	options.arrayMerge = options.arrayMerge || defaultArrayMerge;
	options.isMergeableObject = options.isMergeableObject || isMergeableObject;
	// cloneUnlessOtherwiseSpecified is added to `options` so that custom arrayMerge()
	// implementations can use it. The caller may not replace it.
	options.cloneUnlessOtherwiseSpecified = cloneUnlessOtherwiseSpecified;

	var sourceIsArray = Array.isArray(source);
	var targetIsArray = Array.isArray(target);
	var sourceAndTargetTypesMatch = sourceIsArray === targetIsArray;

	if (!sourceAndTargetTypesMatch) {
		return cloneUnlessOtherwiseSpecified(source, options)
	} else if (sourceIsArray) {
		return options.arrayMerge(target, source, options)
	} else {
		return mergeObject(target, source, options)
	}
}

deepmerge.all = function deepmergeAll(array, options) {
	if (!Array.isArray(array)) {
		throw new Error('first argument should be an array')
	}

	return array.reduce(function(prev, next) {
		return deepmerge(prev, next, options)
	}, {})
};

var deepmerge_1 = deepmerge;

var cjs = deepmerge_1;

const METHODS_WITH_DATA = ['post', 'put', 'patch'];
const createOptions = (method, args) => {
    const dataMethod = METHODS_WITH_DATA.includes(method);
    const optionsIndex = dataMethod ? 2 : args.length - 1;
    const opts = typeof args[optionsIndex] === 'object' ? args[optionsIndex] : {};
    if (method !== 'request' && !opts.method) {
        opts.method = method;
    }
    if (typeof args[0] === 'string') {
        [opts.url] = args;
    }
    if (dataMethod) {
        [, opts.data] = args;
    }
    return Object.assign(Object.assign({}, opts), { toArgs() {
            if (dataMethod) {
                return [this.url, this.data, this];
            }
            return [this.url, this];
        } });
};
function wrapAxiosMethod(sourceAxiosInstance, targetAxiosInstance, method, layerFn) {
    const requestMethod = sourceAxiosInstance[method];
    const wrappedRequest = async (...args) => {
        const options = createOptions(method, args);
        const makeRequest = (updatedOpts) => {
            var _a;
            (_a = updatedOpts) === null || _a === void 0 ? void 0 : _a.toArgs(); // ?
            return requestMethod(...(updatedOpts || options).toArgs());
        };
        return layerFn(makeRequest, options);
    };
    // @ts-ignore
    // eslint-disable-next-line no-param-reassign
    targetAxiosInstance[method] = wrappedRequest;
}
function cloneAxios(axiosInstance) {
    return axios.create(axiosInstance.defaults);
}
const AXIOS_METHODS = ['request', 'get', 'delete', 'head', 'post', 'put', 'patch'];
function wrapAxios(axiosInstance, ...layers) {
    layers.forEach((layer) => AXIOS_METHODS.forEach((method) => wrapAxiosMethod(axiosInstance, axiosInstance, method, layer)));
}
function extendAxios(axiosInstance, ...layers) {
    const axiosClone = cloneAxios(axiosInstance);
    layers.reduce((instance, layer) => {
        AXIOS_METHODS.forEach((method) => wrapAxiosMethod(instance, axiosClone, method, layer));
        return axiosClone;
    }, axiosInstance);
    return axiosClone;
}

const sleep = (time) => new Promise((resolve) => setTimeout(resolve, time));
const pollLayer = ({ delay, retries }) => async (request) => {
    const poll = (retry) => {
        return request().catch((error) => {
            const retriesLeft = retry - 1;
            if (retriesLeft <= 0) {
                throw error;
            }
            return sleep(delay).then(() => poll(retriesLeft));
        });
    };
    return poll(retries);
};

const DEEP_MERGE_PROPS = ['headers', 'auth', 'proxy'];
function createExtendConfigLayer(config) {
    return (makeRequest, options) => {
        const result = Object.assign(Object.assign({}, config), options);
        DEEP_MERGE_PROPS.forEach((prop) => {
            if (config[prop] && options[prop]) {
                result[prop] = cjs(config[prop], options[prop]);
            }
        });
        return makeRequest(result);
    };
}
function addExtendMethod(axiosInstance) {
    const extend = (...args) => {
        const layers = typeof args[0] === 'object' ? [createExtendConfigLayer(args[0])] : args;
        const extendedAxios = extendAxios(axiosInstance, ...layers);
        return addExtendMethod(extendedAxios);
    };
    return Object.assign(axiosInstance, { extend });
}
function create(options) {
    const { layers, config, axiosInstance = axios.create(config) } = options;
    layers.forEach((layer) => wrapAxios(axiosInstance, layer));
    return addExtendMethod(axiosInstance);
}
var index = {
    create,
    wrapAxios,
};

export default index;
export { create, createExtendConfigLayer, extendAxios, pollLayer, wrapAxios };
